@@ -25,25 +25,39 @@
 
 import os
 import json
-from typing import Sequence, Tuple, Mapping, Type
+from typing import Sequence, Tuple, Mapping, Type, List, Optional
 
-from .util import inv_dict, all_subclasses
+from .lntransport import LNPeerAddr
+from .util import inv_dict, all_subclasses, classproperty
 from . import bitcoin
 
 
-def read_json(filename, default):
+def read_json(filename, default=None):
     path = os.path.join(os.path.dirname(__file__), filename)
     try:
         with open(path, 'r') as f:
             r = json.loads(f.read())
     except Exception:
+        if default is None:
+            # Sometimes it's better to hard-fail: the file might be missing
+            # due to a packaging issue, which might otherwise go unnoticed.
+            raise
         r = default
     return r
 
 
+def create_fallback_node_list(fallback_nodes_dict: dict[str, dict]) -> List[LNPeerAddr]:
+    """Take a json dict of fallback nodes like: k:node_id, v:{k:'host', k:'port'} and return LNPeerAddr list"""
+    fallback_nodes = []
+    for node_id, address in fallback_nodes_dict.items():
+        fallback_nodes.append(
+            LNPeerAddr(host=address['host'], port=int(address['port']), pubkey=bytes.fromhex(node_id)))
+    return fallback_nodes
+
+
 GIT_REPO_URL = "https://github.com/spesmilo/electrum"
 GIT_REPO_ISSUES_URL = "https://github.com/spesmilo/electrum/issues"
-BIP39_WALLET_FORMATS = read_json('bip39_wallet_formats.json', [])
+BIP39_WALLET_FORMATS = read_json('bip39_wallet_formats.json')
 
 
 class AbstractNet:
@@ -60,8 +74,6 @@ class AbstractNet:
     BIP44_COIN_TYPE: int
     LN_REALM_BYTE: int
     DEFAULT_PORTS: Mapping[str, str]
-    DEFAULT_SERVERS: Mapping[str, Mapping[str, str]]
-    CHECKPOINTS: Sequence[Tuple[str, int]]
     LN_DNS_SEEDS: Sequence[str]
     XPRV_HEADERS: Mapping[str, int]
     XPRV_HEADERS_INV: Mapping[int, str]
@@ -74,7 +86,54 @@ class AbstractNet:
 
     @classmethod
     def rev_genesis_bytes(cls) -> bytes:
-        return bytes.fromhex(bitcoin.rev_hex(cls.GENESIS))
+        return bytes.fromhex(cls.GENESIS)[::-1]
+
+    @classmethod
+    def set_as_network(cls) -> None:
+        global net
+        net = cls
+
+    _cached_default_servers = None
+    @classproperty
+    def DEFAULT_SERVERS(cls) -> Mapping[str, Mapping[str, str]]:
+        if cls._cached_default_servers is None:
+            default_file = {} if cls.TESTNET else None  # for mainnet we hard-fail if the file is missing.
+            cls._cached_default_servers = read_json(os.path.join('chains', cls.NET_NAME, 'servers.json'), default_file)
+        return cls._cached_default_servers
+
+    _cached_fallback_lnnodes = None
+    @classproperty
+    def FALLBACK_LN_NODES(cls) -> Sequence[LNPeerAddr]:
+        if cls._cached_fallback_lnnodes is None:
+            default_file = {} if cls.TESTNET else None  # for mainnet we hard-fail if the file is missing.
+            d = read_json(os.path.join('chains', cls.NET_NAME, 'fallback_lnnodes.json'), default_file)
+            cls._cached_fallback_lnnodes = create_fallback_node_list(d)
+        return cls._cached_fallback_lnnodes
+
+    _cached_checkpoints = None
+    @classproperty
+    def CHECKPOINTS(cls) -> Sequence[Tuple[str, int]]:
+        if cls._cached_checkpoints is None:
+            default_file = [] if cls.TESTNET else None  # for mainnet we hard-fail if the file is missing.
+            cls._cached_checkpoints = read_json(os.path.join('chains', cls.NET_NAME, 'checkpoints.json'), default_file)
+        return cls._cached_checkpoints
+
+    @classmethod
+    def datadir_subdir(cls) -> Optional[str]:
+        """The name of the folder in the filesystem.
+        None means top-level, used by mainnet.
+        """
+        return cls.NET_NAME
+
+    @classmethod
+    def cli_flag(cls) -> str:
+        """as used in e.g. `$ run_electrum --testnet4`"""
+        return cls.NET_NAME
+
+    @classmethod
+    def config_key(cls) -> str:
+        """as used for SimpleConfig.get()"""
+        return cls.NET_NAME
 
 
 class BitcoinMainnet(AbstractNet):
@@ -88,8 +147,6 @@ class BitcoinMainnet(AbstractNet):
     BOLT11_HRP = SEGWIT_HRP
     GENESIS = "000000000019d6689c085ae165831e934ff763ae46a2a6c172b3f1b60a8ce26f"
     DEFAULT_PORTS = {'t': '50001', 's': '50002'}
-    DEFAULT_SERVERS = read_json('servers.json', {})
-    CHECKPOINTS = read_json('checkpoints.json', [])
     BLOCK_HEIGHT_FIRST_LIGHTNING_CHANNELS = 497000
 
     XPRV_HEADERS = {
@@ -116,6 +173,10 @@ class BitcoinMainnet(AbstractNet):
         'lseed.darosior.ninja',
     ]
 
+    @classmethod
+    def datadir_subdir(cls):
+        return None
+
 
 class BitcoinTestnet(AbstractNet):
 
@@ -128,8 +189,6 @@ class BitcoinTestnet(AbstractNet):
     BOLT11_HRP = SEGWIT_HRP
     GENESIS = "000000000933ea01ad0ee984209779baaec3ced90fa3f408719526f8d77f4943"
     DEFAULT_PORTS = {'t': '51001', 's': '51002'}
-    DEFAULT_SERVERS = read_json('servers_testnet.json', {})
-    CHECKPOINTS = read_json('checkpoints_testnet.json', [])
 
     XPRV_HEADERS = {
         'standard':    0x04358394,  # tprv
@@ -155,14 +214,19 @@ class BitcoinTestnet(AbstractNet):
     ]
 
 
+class BitcoinTestnet4(BitcoinTestnet):
+
+    NET_NAME = "testnet4"
+    GENESIS = "00000000da84f2bafbbc53dee25a72ae507ff4914b867c565be350b0da8bf043"
+    LN_DNS_SEEDS = []
+
+
 class BitcoinRegtest(BitcoinTestnet):
 
     NET_NAME = "regtest"
     SEGWIT_HRP = "bcrt"
     BOLT11_HRP = SEGWIT_HRP
     GENESIS = "0f9188f13cb7b2c71f2a335e3a4fc328bf5beb436012afca590b1a11466e2206"
-    DEFAULT_SERVERS = read_json('servers_regtest.json', {})
-    CHECKPOINTS = []
     LN_DNS_SEEDS = []
 
 
@@ -175,8 +239,6 @@ class BitcoinSimnet(BitcoinTestnet):
     SEGWIT_HRP = "sb"
     BOLT11_HRP = SEGWIT_HRP
     GENESIS = "683e86bd5c6d110d91b94b97137ba6bfe02dbbdb8e3dff722a669b5d69d77af6"
-    DEFAULT_SERVERS = read_json('servers_regtest.json', {})
-    CHECKPOINTS = []
     LN_DNS_SEEDS = []
 
 
@@ -185,32 +247,16 @@ class BitcoinSignet(BitcoinTestnet):
     NET_NAME = "signet"
     BOLT11_HRP = "tbs"
     GENESIS = "00000008819873e925422c1ff0f99f7cc9bbb232af63a077a480a3633bee1ef6"
-    DEFAULT_SERVERS = read_json('servers_signet.json', {})
-    CHECKPOINTS = []
     LN_DNS_SEEDS = []
 
 
-NETS_LIST = tuple(all_subclasses(AbstractNet))
+NETS_LIST = tuple(all_subclasses(AbstractNet))  # type: Sequence[Type[AbstractNet]]
+NETS_LIST = tuple(sorted(NETS_LIST, key=lambda x: x.NET_NAME))
+
+assert len(NETS_LIST) == len(set([chain.NET_NAME for chain in NETS_LIST])), "NET_NAME must be unique for each concrete AbstractNet"
+assert len(NETS_LIST) == len(set([chain.datadir_subdir() for chain in NETS_LIST])), "datadir must be unique for each concrete AbstractNet"
+assert len(NETS_LIST) == len(set([chain.cli_flag() for chain in NETS_LIST])), "cli_flag must be unique for each concrete AbstractNet"
+assert len(NETS_LIST) == len(set([chain.config_key() for chain in NETS_LIST])), "config_key must be unique for each concrete AbstractNet"
 
 # don't import net directly, import the module instead (so that net is singleton)
 net = BitcoinMainnet  # type: Type[AbstractNet]
-
-def set_signet():
-    global net
-    net = BitcoinSignet
-
-def set_simnet():
-    global net
-    net = BitcoinSimnet
-
-def set_mainnet():
-    global net
-    net = BitcoinMainnet
-
-def set_testnet():
-    global net
-    net = BitcoinTestnet
-
-def set_regtest():
-    global net
-    net = BitcoinRegtest
