@@ -14,18 +14,8 @@
 # TODO impl ADDR descriptors
 # TODO impl RAW descriptors
 
-import enum
-
-from .bip32 import convert_bip32_strpath_to_intpath, BIP32Node, KeyOriginInfo, BIP32_PRIME
-from . import bitcoin
-from .bitcoin import construct_script, opcodes, construct_witness
-from . import constants
-from .crypto import hash_160, sha256
-from . import ecc
-from . import segwit_addr
-from .util import bfh
-
 from binascii import unhexlify
+import enum
 from enum import Enum
 from typing import (
     List,
@@ -35,10 +25,20 @@ from typing import (
     Sequence,
     Mapping,
     Set,
+    Union,
 )
 
+import electrum_ecc as ecc
 
-MAX_TAPROOT_NODES = 128
+from .bip32 import convert_bip32_strpath_to_intpath, BIP32Node, KeyOriginInfo, BIP32_PRIME
+from . import bitcoin
+from .bitcoin import construct_script, opcodes, construct_witness, taproot_output_script
+from . import constants
+from .crypto import hash_160, sha256
+from . import segwit_addr
+
+
+MAX_TAPROOT_DEPTH = 128
 
 # we guess that signatures will be 72 bytes long
 # note: DER-encoded ECDSA signatures are 71 or 72 bytes in practice
@@ -75,7 +75,7 @@ class ExpandedScripts:
         self._scriptcode_for_sighash = value
 
     def address(self, *, net=None) -> Optional[str]:
-        return bitcoin.script_to_address(self.output_script.hex(), net=net)
+        return bitcoin.script_to_address(self.output_script, net=net)
 
 
 class ScriptSolutionInner(NamedTuple):
@@ -379,9 +379,9 @@ class Descriptor(object):
         witness = None
         script_sig = None
         if self.is_segwit():
-            witness = bfh(construct_witness(sol.witness_items))
+            witness = construct_witness(sol.witness_items)
         else:
-            script_sig = bfh(construct_script(sol.witness_items))
+            script_sig = construct_script(sol.witness_items)
         return ScriptSolutionTop(
             witness=witness,
             script_sig=script_sig,
@@ -415,6 +415,9 @@ class Descriptor(object):
     def is_segwit(self) -> bool:
         return any([desc.is_segwit() for desc in self.subdescriptors])
 
+    def is_taproot(self) -> bool:
+        return False
+
     def get_all_pubkeys(self) -> Set[bytes]:
         """Returns set of pubkeys that appear at any level in this descriptor."""
         assert not self.is_range()
@@ -473,7 +476,7 @@ class PKDescriptor(Descriptor):
     def expand(self, *, pos: Optional[int] = None) -> "ExpandedScripts":
         pubkey = self.pubkeys[0].get_pubkey_bytes(pos=pos)
         script = construct_script([pubkey, opcodes.OP_CHECKSIG])
-        return ExpandedScripts(output_script=bytes.fromhex(script))
+        return ExpandedScripts(output_script=script)
 
     def _satisfy_inner(self, *, sigdata=None, allow_dummy=False) -> ScriptSolutionInner:
         if sigdata is None: sigdata = {}
@@ -513,9 +516,9 @@ class PKHDescriptor(Descriptor):
 
     def expand(self, *, pos: Optional[int] = None) -> "ExpandedScripts":
         pubkey = self.pubkeys[0].get_pubkey_bytes(pos=pos)
-        pkh = hash_160(pubkey).hex()
+        pkh = hash_160(pubkey)
         script = bitcoin.pubkeyhash_to_p2pkh_script(pkh)
-        return ExpandedScripts(output_script=bytes.fromhex(script))
+        return ExpandedScripts(output_script=script)
 
     def _satisfy_inner(self, *, sigdata=None, allow_dummy=False) -> ScriptSolutionInner:
         if sigdata is None: sigdata = {}
@@ -556,10 +559,10 @@ class WPKHDescriptor(Descriptor):
     def expand(self, *, pos: Optional[int] = None) -> "ExpandedScripts":
         pkh = hash_160(self.pubkeys[0].get_pubkey_bytes(pos=pos))
         output_script = construct_script([0, pkh])
-        scriptcode = bitcoin.pubkeyhash_to_p2pkh_script(pkh.hex())
+        scriptcode = bitcoin.pubkeyhash_to_p2pkh_script(pkh)
         return ExpandedScripts(
-            output_script=bytes.fromhex(output_script),
-            scriptcode_for_sighash=bytes.fromhex(scriptcode),
+            output_script=output_script,
+            scriptcode_for_sighash=scriptcode,
         )
 
     def _satisfy_inner(self, *, sigdata=None, allow_dummy=False) -> ScriptSolutionInner:
@@ -625,7 +628,7 @@ class MultisigDescriptor(Descriptor):
         der_pks = [p.get_pubkey_bytes(pos=pos) for p in self.pubkeys]
         if self.is_sorted:
             der_pks.sort()
-        script = bfh(construct_script([self.thresh, *der_pks, len(der_pks), opcodes.OP_CHECKMULTISIG]))
+        script = construct_script([self.thresh, *der_pks, len(der_pks), opcodes.OP_CHECKMULTISIG])
         return ExpandedScripts(output_script=script)
 
     def _satisfy_inner(self, *, sigdata=None, allow_dummy=False) -> ScriptSolutionInner:
@@ -678,7 +681,7 @@ class SHDescriptor(Descriptor):
         sub_scripts = self.subdescriptors[0].expand(pos=pos)
         redeem_script = sub_scripts.output_script
         witness_script = sub_scripts.witness_script
-        script = bfh(construct_script([opcodes.OP_HASH160, hash_160(redeem_script), opcodes.OP_EQUAL]))
+        script = construct_script([opcodes.OP_HASH160, hash_160(redeem_script), opcodes.OP_EQUAL])
         return ExpandedScripts(
             output_script=script,
             redeem_script=redeem_script,
@@ -697,10 +700,10 @@ class SHDescriptor(Descriptor):
         witness = None
         if isinstance(subdesc, (WSHDescriptor, WPKHDescriptor)):  # witness_v0 nested in p2sh
             witness = subdesc.satisfy(sigdata=sigdata, allow_dummy=allow_dummy).witness
-            script_sig = bfh(construct_script([redeem_script]))
+            script_sig = construct_script([redeem_script])
         else:  # legacy p2sh
             subsol = subdesc._satisfy_inner(sigdata=sigdata, allow_dummy=allow_dummy)
-            script_sig = bfh(construct_script([*subsol.witness_items, redeem_script]))
+            script_sig = construct_script([*subsol.witness_items, redeem_script])
         return ScriptSolutionTop(
             witness=witness,
             script_sig=script_sig,
@@ -724,7 +727,7 @@ class WSHDescriptor(Descriptor):
         assert len(self.subdescriptors) == 1
         sub_scripts = self.subdescriptors[0].expand(pos=pos)
         witness_script = sub_scripts.output_script
-        output_script = bfh(construct_script([0, sha256(witness_script)]))
+        output_script = construct_script([0, sha256(witness_script)])
         return ExpandedScripts(
             output_script=output_script,
             witness_script=witness_script,
@@ -740,7 +743,7 @@ class WSHDescriptor(Descriptor):
         witness_script = self.expand().witness_script
         witness = construct_witness([*subsol.witness_items, witness_script])
         return ScriptSolutionTop(
-            witness=bytes.fromhex(witness),
+            witness=witness,
         )
 
     def is_segwit(self) -> bool:
@@ -754,43 +757,78 @@ class TRDescriptor(Descriptor):
     def __init__(
         self,
         internal_key: 'PubkeyProvider',
-        subdescriptors: List['Descriptor'] = None,
-        depths: List[int] = None,
+        desc_tree: List[Union['Descriptor', List]] = None,
     ) -> None:
         r"""
         :param internal_key: The :class:`PubkeyProvider` that is the internal key for this descriptor
-        :param subdescriptors: The :class:`Descriptor`\ s that are the leaf scripts for this descriptor
-        :param depths: The depths of the leaf scripts in the same order as `subdescriptors`
+        :param desc_tree: Taproot script binary tree, as a nested list of Descriptors
         """
-        if subdescriptors is None:
-            subdescriptors = []
-        if depths is None:
-            depths = []
-        super().__init__([internal_key], subdescriptors, "tr")
-        self.depths = depths
+        if desc_tree is None:
+            desc_tree = []
+        self.desc_tree = desc_tree
+        desc_list = []
+        if desc_tree:
+            if self.get_max_tree_depth() > MAX_TAPROOT_DEPTH:
+                raise ValueError(f"tr() supports at most {MAX_TAPROOT_DEPTH} nesting levels")
+            def flatten(tree_node):
+                if isinstance(tree_node, Descriptor):
+                    return [tree_node]
+                assert len(tree_node) == 2, len(tree_node)
+                return flatten(tree_node[0]) + flatten(tree_node[1])
+            desc_list = flatten(desc_tree)
+        super().__init__(
+            pubkeys=[internal_key],
+            subdescriptors=desc_list,  # FIXME we could do without the flattened list (dupl)
+            name="tr",
+        )
 
     def to_string_no_checksum(self) -> str:
-        r = f"{self.name}({self.pubkeys[0].to_string()}"
-        path: List[bool] = [] # Track left or right for each depth
-        for p, depth in enumerate(self.depths):
-            r += ","
-            while len(path) <= depth:
-                if len(path) > 0:
-                    r += "{"
-                path.append(False)
-            r += self.subdescriptors[p].to_string_no_checksum()
-            while len(path) > 0 and path[-1]:
-                if len(path) > 0:
-                    r += "}"
-                path.pop()
-            if len(path) > 0:
-                path[-1] = True
-        r += ")"
-        return r
+        ret = f"{self.name}({self.pubkeys[0].to_string()}"
+        if self.desc_tree:
+            ret += ","
+            def tree_to_str(tree_node):
+                if isinstance(tree_node, Descriptor):
+                    return tree_node.to_string_no_checksum()
+                assert len(tree_node) == 2, len(tree_node)
+                return "{" + tree_to_str(tree_node[0]) + "," + tree_to_str(tree_node[1]) + "}"
+            ret += tree_to_str(self.desc_tree)
+        ret += ")"
+        return ret
 
     def is_segwit(self) -> bool:
         return True
 
+    def is_taproot(self) -> bool:
+        return True
+
+    # TODO add more test vectors from BIP-0386
+    def expand(self, *, pos: Optional[int] = None) -> "ExpandedScripts":
+        internal_pubkey = self.pubkeys[0].get_pubkey_bytes(pos=pos)
+        script_tree = None
+        if self.desc_tree:
+            def transform(tree_node):
+                if isinstance(tree_node, Descriptor):
+                    leaf_version = 0xc0
+                    leaf_script = tree_node.expand(pos=pos).scriptcode_for_sighash  # FIXME maybe rename scriptcode_for_sighash
+                    return (leaf_version, leaf_script)
+                assert len(tree_node) == 2, len(tree_node)
+                return [transform(tree_node[0]), transform(tree_node[1])]
+            script_tree = transform(self.desc_tree)
+        output_script = taproot_output_script(internal_pubkey, script_tree=script_tree)
+        return ExpandedScripts(
+            output_script=output_script,
+        )
+
+    def get_max_tree_depth(self) -> Optional[int]:
+        if not self.desc_tree:
+            return None
+        def depth(tree_node) -> int:
+            if isinstance(tree_node, Descriptor):
+                return 0
+            assert len(tree_node) == 2, len(tree_node)
+            return 1 + max(depth(tree_node[0]), depth(tree_node[1]))
+        return depth(self.desc_tree)
+
 
 def _get_func_expr(s: str) -> Tuple[str, str]:
     """
@@ -800,9 +838,12 @@ def _get_func_expr(s: str) -> Tuple[str, str]:
     :return: The function name as the first element of the tuple, and the expression contained within the function as the second element
     :raises: ValueError: if a matching pair of parentheses cannot be found
     """
-    start = s.index("(")
-    end = s.rindex(")")
-    return s[0:start], s[start + 1:end]
+    try:
+        start = s.index("(")
+        end = s.rindex(")")
+        return s[0:start], s[start + 1:end]
+    except ValueError:
+        raise ValueError("A matching pair of parentheses cannot be found")
 
 
 def _get_const(s: str, const: str) -> str:
@@ -838,6 +879,8 @@ def _get_expr(s: str) -> Tuple[str, str]:
             level -= 1
         elif level == 0 and c in [")", "}", ","]:
             break
+    else:
+        return s, ""
     return s[0:i], s[i:]
 
 def parse_pubkey(expr: str, *, ctx: '_ParseDescriptorContext') -> Tuple['PubkeyProvider', str]:
@@ -941,39 +984,24 @@ def _parse_descriptor(desc: str, *, ctx: '_ParseDescriptorContext') -> 'Descript
         if ctx != _ParseDescriptorContext.TOP:
             raise ValueError("Can only have tr at top level")
         internal_key, expr = parse_pubkey(expr, ctx=ctx)
-        subscripts = []
-        depths = []
+        desc_tree = []
         if expr:
-            # Path from top of the tree to what we're currently processing.
-            # branches[i] == False: left branch in the i'th step from the top
-            # branches[i] == true: right branch
-            branches = []
-            while True:
-                # Process open braces
-                while True:
-                    try:
-                        expr = _get_const(expr, "{")
-                        branches.append(False)
-                    except ValueError:
-                        break
-                    if len(branches) > MAX_TAPROOT_NODES:
-                        raise ValueError(f"tr() supports at most {MAX_TAPROOT_NODES} nesting levels")  # TODO xxxx fixed upstream bug here
-                # Process script expression
-                sarg, expr = _get_expr(expr)
-                subscripts.append(_parse_descriptor(sarg, ctx=_ParseDescriptorContext.P2TR))
-                depths.append(len(branches))
-                # Process closing braces
-                while len(branches) > 0 and branches[-1]:
-                    expr = _get_const(expr, "}")
-                    branches.pop()
-                # If we're at the end of a left branch, expect a comma
-                if len(branches) > 0 and not branches[-1]:
-                    expr = _get_const(expr, ",")
-                    branches[-1] = True
-
-                if len(branches) == 0:
-                    break
-        return TRDescriptor(internal_key, subscripts, depths)
+            def parse_tree(tree_str):
+                if len(tree_str) == 0:
+                    raise ValueError("Invalid Taproot tree expression")
+                if tree_str[0] != "{":  # leaf
+                    sarg, remaining = _get_expr(tree_str)
+                    return _parse_descriptor(sarg, ctx=_ParseDescriptorContext.P2TR), remaining
+                if len(tree_str) < len("{x,y}") or tree_str[-1] != "}":
+                    raise ValueError("Invalid Taproot tree expression")
+                left, remaining = parse_tree(tree_str[1:])
+                if remaining[0] != ",": raise ValueError
+                right, remaining = parse_tree(remaining[1:])
+                if remaining[0] != "}": raise ValueError
+                return [left, right], remaining[1:]
+            desc_tree, _remaining = parse_tree(expr)
+            if len(_remaining) != 0: raise ValueError
+        return TRDescriptor(internal_key, desc_tree)
     if ctx == _ParseDescriptorContext.P2SH:
         raise ValueError("A function is needed within P2SH")
     elif ctx == _ParseDescriptorContext.P2WSH:
