@@ -586,13 +586,22 @@ class TrustedCoinPlugin(BasePlugin):
                         else 'trustedcoin_have_seed'
             },
             'trustedcoin_create_seed': {
-                'next': 'trustedcoin_confirm_seed'
+                'next': lambda d: 'trustedcoin_create_ext' if wizard.wants_ext(d) else 'trustedcoin_confirm_seed',
+            },
+            'trustedcoin_create_ext': {
+                'next': 'trustedcoin_confirm_seed',
             },
             'trustedcoin_confirm_seed': {
-                'next': 'trustedcoin_tos'
+                'next': lambda d: 'trustedcoin_confirm_ext' if wizard.wants_ext(d) else 'trustedcoin_tos',
+            },
+            'trustedcoin_confirm_ext': {
+                'next': 'trustedcoin_tos',
             },
             'trustedcoin_have_seed': {
-                'next': 'trustedcoin_keep_disable'
+                'next': lambda d: 'trustedcoin_have_ext' if wizard.wants_ext(d) else 'trustedcoin_keep_disable',
+            },
+            'trustedcoin_have_ext': {
+                'next': 'trustedcoin_keep_disable',
             },
             'trustedcoin_keep_disable': {
                 'next': lambda d: 'trustedcoin_tos' if d['trustedcoin_keepordisable'] != 'disable'
